{
  "type": "record",
  "name": "Struct",
  "fields": [
    {
      "name": "value",
      "type": {
        "type": "map",
        "values": [
          "null",
          "boolean",
          { "type": "map", "values": "Struct" },
          { "type": "array", "items": "Struct" },
          "double",
          "string"
        ]
      }
    }
  ]
}
